#include <avr/io.h>
#include <avr/interrupt.h>

#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>

#include "lcd.h"

void timer2_init() {
	TCCR2B |= _BV(CS20);
	TCCR2A |= _BV(WGM21) | _BV(WGM20) | _BV(COM2A1) | _BV(COM2B1);
	OCR2A = 0; // max value
	OCR2B = 0;
	
	DDRD |= _BV(PD7) | _BV(PD6);
}

void timer1_init()	{
	TCCR1B |= _BV(CS10) | _BV(WGM13) | _BV(WGM12);
	TCCR1A |= _BV(WGM11) | _BV(COM1B1) | _BV(COM1A1);
	ICR1 = 50000;
	// duty cycle-ul este setat initial la 0%
	OCR1B = 0;
	OCR1A = 0;
}

void timer0_init() {
	TCCR0B = (1 << CS00);
	TCCR0A |= _BV(WGM01) | _BV(WGM00) | _BV(COM0A1) | _BV(COM0B1);
	
	OCR0A = 0;
	OCR0B = 0;
	
	DDRB |= _BV(PB3) | _BV(PB4);
}

void IO_init() {
	DDRC &= ~_BV(PC0);// butonul e pus ca input
	PORTC |= _BV(PC0);// activez rezistenta de pull-up
	
	DDRA &= ~_BV(PA7);
	PORTA |= _BV(PA7);
}

void display_duty_cycle(int selected, int increase, int clicks[5]) {
	int limit = 0;
	char dbg[16];
	
	if (increase == 1) {
		clicks[selected - 1]++;
		if (clicks[selected - 1] > 20) {
			clicks[selected - 1] = 0;
		}
	}
	
	int clicks_made = clicks[selected - 1];
	
	switch (selected) {
		case 1:
			// timer 0, pin PB3
			limit = 255;
			OCR0A = (clicks_made * 5 * limit) / 100;
			break;
		case 2:
			// timer 0, pin PB4
			limit = 255;
			OCR0B = (clicks_made * 5 * limit) / 100;
			break;
		case 3:
			// timer 2, pin PD7
			limit = 255;
			OCR2A = (clicks_made * 5 * limit) / 100;
			break;
		case 4:
			// timer 2, pin PD6
			limit = 255;
			OCR2B = (clicks_made * 5 * limit) / 100;
			break;
		case 5:
			// timer 1, pin PD5
			limit = ICR1;
			OCR1A = (clicks_made * 5 * limit) / 100;
			break;
	}
	
	LCD_clear_top_line();
	sprintf(dbg, "I: %d%%", (clicks_made * 5));
	LCD_printAt(0x00, dbg);
}
 
int main(void) {
	/*
	 * Timer 0:
	 * OC0A = PB3
	 * OC0B = PB4
	 
	 * Timer 1:
	 * OC1A = PD5
	 * OC1B = PD4
	 
	 * Timer 2:
	 * OC2A = PD7
	 * OC2B = PD6
	 
	 * Timer 3:
	 * OC3A = PB6
	 * OC3B = PB7
	 *
	 */
	int button_pushed = 0;
	int button_two_pushed = 0;
	int step;
	
	timer2_init();
	timer1_init();
	timer0_init();
	IO_init();
	
	LCD_init();
	
	int selected = 1;
	char dbg[16];
	int pcl1 = 0;
	int rcl1 = 0;
	int pcl2 = 0;
	int rcl2 = 0;
	
	step = 5 * 255 / 100;
	int clicks[5];
	clicks[0] = 0;
	clicks[1] = 0;
	clicks[2] = 0;
	clicks[3] = 0;
	clicks[4] = 0;
	
	display_duty_cycle(selected, 0, clicks);
	LCD_printAt(0x40, "Canal: 1");
 
	while (1)
	{
		if (bit_is_clear(PINC, PC0)) {
			pcl1++;
			if (pcl1 > 500) {
				if (button_pushed == 0) {
					display_duty_cycle(selected, 1, clicks);
					button_pushed = 1;
				}
				pcl1 = 0;
			}
		}
		else if (!bit_is_clear(PINC, PC0)) {
			rcl1++;
			if (rcl1 > 500) {
				button_pushed = 0;
				rcl1 = 0;
			}
		}
		
		if (bit_is_clear(PINA, PA7)) {
			pcl2++;
			if (pcl2 > 500) {
				if (button_two_pushed == 0) {
					selected += 1;
					if (selected > 5) {
						selected = 1;
					}
					display_duty_cycle(selected, 0, clicks);
					sprintf(dbg, "Canal: %d", selected);
					LCD_printAt(0x40, dbg);
					button_two_pushed = 1;
				}
				pcl2 = 0;
			}
		}
		else if (!bit_is_clear(PINA, PA7)) {
			rcl2++;
			if (rcl2 > 500) {
				button_two_pushed = 0;
				rcl2 = 0;
			}
		}
	}  
	return 0;
}